/*-----------------------------------------------------------------------------
 * CS 155 - Computer & Network Security - Spring 2004
 *
 * file:  inject.c
 * Contact: pkpatel@cs.stanford.edu
 *          mrubens@stanford.edu
 *
 * Description:
 * Attacker for RLOGIN packet sniffing & packet injection attack
 *
 *---------------------------------------------------------------------------*/

#include "inject.h"

int main(int argc, char *argv[])
{
  int retval;

  if (getuid() != 0) {
    fprintf(stderr, "This program must be run as root!\n");
    exit(-1);
  }
  
  retval = capture();
  return retval;
}

int capture()
{
  pcap_t                    *handle;          /* Session handle */
  char 		            *dev;             /* The device to sniff on */
  char 		            errbuf[PCAP_ERRBUF_SIZE]; /*Err string */
  bpf_u_int32               mask;             /* Our netmask */
  bpf_u_int32               net;              /* Our IP */
  struct pcap_pkthdr        header;           /* Header that pcap gives us */
  const u_char              *packet;          /* The actual packet */
  
  const struct ether_header *ether_header;    /* The Ethernet header */
  const struct ip           *ip;              /* The IP header */
  const struct tcphdr       *tcphdr;          /* The TCP header */
  const char                *payload;         /* Packet payload */
  
  struct spoof_info         *spoof_data = NULL;
  
  /* Used for debugging purposes */
  char                      *proto;
  u_short                   src_port;
  u_short                   dst_port;
  
  /* Sizes of some relevant headers */
  int size_ethernet       = sizeof(struct ether_header); 
  int size_ip             = sizeof(struct ip);
  int size_tcp            = sizeof(struct tcphdr);
  
  spoof_data = (struct spoof_info *)malloc(sizeof(struct spoof_info));
  if(!spoof_data)
  {
    return(-1);
  }
  
  /* Setup Pcap - packet capture interface */
  /* Define the device */
  dev = pcap_lookupdev(errbuf);
  printf("Listening on %s.\n", dev);
  
  /* Find the properties for the device */
  pcap_lookupnet(dev, &net, &mask, errbuf);
  
  /* Open the session in promiscuous mode */
  handle = pcap_open_live(dev, BUFSIZ, 1, 0, errbuf);
  
  /* You decide when to break */
  while (1)
  {
    packet = pcap_next(handle, &header);
    
    /* Get the Ethernet header */
    ether_header = (struct ether_header*)(packet);
    spoof_data->srcenet = (u_char*)ether_header->ether_shost;
    spoof_data->dstenet = (u_char*)ether_header->ether_dhost;
    
#ifdef DEBUG
    printf("Ethernet: %02x.%02x.%02x.%02x.%02x.%02x --> ",
	   spoof_data->srcenet[0],
	   spoof_data->srcenet[1],
	   spoof_data->srcenet[2],
	   spoof_data->srcenet[3],
	   spoof_data->srcenet[4],
	   spoof_data->srcenet[5]);
    
    printf("%02x.%02x.%02x.%02x.%02x.%02x\n",
	   spoof_data->dstenet[0],
	   spoof_data->dstenet[1],
	   spoof_data->dstenet[2],
	   spoof_data->dstenet[3],
	   spoof_data->dstenet[4],
	   spoof_data->dstenet[5]);
#endif
    
    /* If we have an IP packet... */
    if (ntohs(ether_header->ether_type) == ETHERTYPE_IP) 
    {
      /* Get the IP header */
      ip = (struct ip*)(packet + size_ethernet);
      
#ifdef DEBUG
      /* Print some info for testing */
      printf("SrcIP : %s-->", inet_ntoa(ip->ip_src));
      printf("DstIP : %s\n\n", inet_ntoa(ip->ip_dst));
#endif

      /* If we have a TCP packet... */
      if (ip->ip_p == IPPROTO_TCP)
      {
	  proto = "ip:tcp";
	  
	  tcphdr = (struct tcphdr*)(packet + size_ethernet + size_ip);

	  src_port = ntohs(tcphdr->source);
	  dst_port = ntohs(tcphdr->dest);

	  /*
	   * Determine if this a useful packet
	   * Gather the spoofing data
	   * Call inject when done
	   */
	  
	  /* inject_packet(spoof_data); */
      }
    }
  }
  /* And close the session */
  pcap_close(handle);
  return(0);
}


/*
 * Packet injection using Libnet
 */
void inject_packet(struct spoof_info *spoof_data)
{
  int count;
  libnet_t *l;
  libnet_ptag_t t;
  char *payload;
  u_short payload_s;
  char errbuf[LIBNET_ERRBUF_SIZE];
  
  printf("Creating packet to inject\n");
  
  /*
   *  Initialize the library.  Root priviledges are required.
   */
  l = libnet_init(
            LIBNET_LINK,                    /* injection type */
            NULL,                           /* network interface */
            errbuf);                        /* error buffer */

  if (l == NULL)
  {
    fprintf(stderr, "libnet_init() failed: %s", errbuf);
    exit(EXIT_FAILURE);
  }

  /* Get the TCP payload - for us its our EGG */
  
  /* 
   * Take care of the TCP options first
   * libnet_build_tcp_options(...);
   */
  
  /* 
   * Now build the TCP header
   * libnet_build_tcp(...);
   */
  
  /* 
   * ... the IP header
   * libnet_build_ipv4(...);
   */
  
  /* 
   * Ethernet hdr - almost done
   * libnet_build_ethernet(...);
   */
  
  /*
   * Write it to the wire - boooom!
   * libnet_write(l);
   */
  
  libnet_destroy(l);
  return;
}
