/*-----------------------------------------------------------------------------
 * CS 155 - Computer & Network Security - Spring 2004
 *
 * file:  sniff.c
 * Contact: pkpatel@cs.stanford.edu
 *          mrubens@stanford.edu
 *
 * Description:
 * Attacker for FTP file sniffing
 *
 *---------------------------------------------------------------------------*/

#include "sniff.h"

int main(int argc, char *argv[])
{
  int retval;
  
  if (getuid() != 0) {
    fprintf(stderr, "This program must be run as root!\n");
    exit(-1);
  }
  
  retval = capture();
  return retval;
}

int capture()
{
  pcap_t                     *handle;		/* Session handle */
  char 			     *dev;	       	/* The device to sniff on */
  char 			     errbuf[PCAP_ERRBUF_SIZE]; /*Err string */
  bpf_u_int32 		     mask;	       	/* Our netmask */
  bpf_u_int32 		     net;	       	/* Our IP */
  struct pcap_pkthdr 	     header;           	/* Header that pcap gives us */
  const u_char 		     *packet;		/* The actual packet */
  
  const struct ether_header  *ether_header; 	/* The Ethernet header */
  const struct ip 	     *ip; 	       	/* The IP header */
  const struct tcphdr 	     *tcphdr;   	/* The TCP header */
  const char 		     *payload; 		/* Packet payload */
  
  /* Used for debugging purposes */
  char 			     *proto;
  u_char 		     *srcenet;
  u_char		     *dstenet;
  u_short 		     src_port;
  u_short 		     dst_port;
  
  /* Sizes of some relevant headers */
  int size_ethernet 	   = sizeof(struct ether_header); 
  int size_ip 		   = sizeof(struct ip);
  int size_tcp 		   = sizeof(struct tcphdr);
  
  /* Setup Pcap - packet capture interface */
  /* Define the device */
  dev = pcap_lookupdev(errbuf);
  printf("Listening on %s.\n", dev);
  
  /* Find the properties for the device */
  pcap_lookupnet(dev, &net, &mask, errbuf);
  
  /* Open the session in promiscuous mode */
  handle = pcap_open_live(dev, BUFSIZ, 1, 0, errbuf);
  
  /* When do you want to exit? */
  while (1)
  {
    packet = pcap_next(handle, &header);
    
    /* Get the Ethernet header */
    ether_header = (struct ether_header*)(packet);
    srcenet = (u_char*)ether_header->ether_shost;
    dstenet = (u_char*)ether_header->ether_dhost;
    
#ifdef DEBUG
    printf("Ethernet: %02x.%02x.%02x.%02x.%02x.%02x --> ",
	   srcenet[0],
	   srcenet[1],
	   srcenet[2],
	   srcenet[3],
	   srcenet[4],
	   srcenet[5]);
    
    printf("%02x.%02x.%02x.%02x.%02x.%02x\n",
	   dstenet[0],
	   dstenet[1],
	   dstenet[2],
	   dstenet[3],
	   dstenet[4],
	   dstenet[5]);
#endif
    
    /* If we have an IP packet... */
    if (ntohs(ether_header->ether_type) == ETHERTYPE_IP) 
    {
      /* Get the IP header */
      ip = (struct ip*)(packet + size_ethernet);
      
#ifdef DEBUG
      /* Print some info for testing */
      printf("SrcIP : %s-->", inet_ntoa(ip->ip_src));
      printf("DstIP : %s\n\n", inet_ntoa(ip->ip_dst));
#endif
      
      /* If we have a TCP packet... */
      if (ip->ip_p == IPPROTO_TCP) 
      {
	tcphdr = (struct tcphdr*)(packet + size_ethernet + size_ip);

	src_port = ntohs(tcphdr->source);
	dst_port = ntohs(tcphdr->dest);

	/*
	 * Determine if this a useful packet
	 * If yes, grab data and write to our file
	 */
	
      }
    }
  }
  /* And close the session */
  pcap_close(handle);
  return(0);
}
